#include <jmp-config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#ifdef SUPPORT_PTHREAD
 #include <pthread.h>
#endif

#include <jmp.h>
#include <comparators.h>
#include <deadlock_detector.h>

#include <ui_file.h>

/** This file implements the ui interface. 
 *  The ui provided by this file is: none, dumping to file works.
 *
 *  Some methods are marked with "This method does nothing." 
 *  Thoose methods are only here to enable the no-ui to link correctly.
 */

static int quit;
#ifdef SUPPORT_PTHREAD
static pthread_mutex_t mutex_quit;
static pthread_cond_t cond_quit;
#endif

static int checkShutdownUI (void) {
    int rv = 0;
    PTHREAD_MUTEX_LOCK (&mutex_quit);
    if (quit != 0)
        rv = 1;
    PTHREAD_MUTEX_UNLOCK (&mutex_quit);
    return rv;
}

static int checkThenDoShutdownUI (void) {
    int rv = 0;
    PTHREAD_MUTEX_LOCK (&mutex_quit);
    if (quit == 1) {
        /* time to shut down. */
        quit = 2;
        rv = 1;
    } else if (quit != 0) {
        rv = 1;
    }
    PTHREAD_MUTEX_UNLOCK (&mutex_quit);
    return rv;
}

static int ui_state (int below, int set) {
    int rv = 0;
    PTHREAD_MUTEX_LOCK (&mutex_quit);
    if (quit <= below) {
        if (quit == below) {
            quit = set;
            PTHREAD_COND_BROADCAST (&cond_quit);
            rv = 1;
        }
    }
    PTHREAD_MUTEX_UNLOCK (&mutex_quit);
    return rv;
}

int run_updates (void* data) {
    if (checkThenDoShutdownUI())
        return 0;

    dumptimer_tick();

    return 1;
}

void end_ui (void) {
    PTHREAD_COND_DESTROY (&cond_quit);
    PTHREAD_MUTEX_DESTROY (&mutex_quit);
}

void init_ui (void) {
    PTHREAD_MUTEX_INIT (&mutex_quit, NULL);
    PTHREAD_COND_INIT (&cond_quit, NULL);
    quit = 0;
}

void start_ui (void) {
    /* do nothing. */
}

void stop_ui (void) {
    /* do nothing. */
}

void ask_stop_ui (void) {
    /* do nothing. */
}

void gtkthread (void* data) {
    int loop = 1;
    PTHREAD_MUTEX_LOCK (&mutex_quit);
    while (loop) {
        struct timespec timespec;
        struct timeval timeval;

        PTHREAD_MUTEX_UNLOCK (&mutex_quit);
	run_updates (NULL);

        gettimeofday(&timeval, NULL);
        timespec.tv_sec = timeval.tv_sec + 1;
        timespec.tv_nsec = timeval.tv_usec * 1000;
        PTHREAD_MUTEX_LOCK (&mutex_quit);
        for (;;) {
            if (quit >= 1) {
                loop = 0;
                quit = 3;
                PTHREAD_COND_BROADCAST (&cond_quit);
                break;
            }
            PTHREAD_COND_TIMEDWAIT (&cond_quit, &mutex_quit, &timespec);
        }
    }
    PTHREAD_MUTEX_UNLOCK (&mutex_quit);
    ui_state (3, 4);
}

/** Tell the ui system to shut itself down. */
void quit_ui (void) {
    quit = 1;
}

/** Update the statistics. */
void updateUI (hashtab* cls, hashtab* methods) {
}


/** Set the status text... */
void set_status (char* text) {
}

/** methods used to show a window with owners to objects of the given type.
 *  This method does nothing.
 * @param objects the hash table of objects to look in.
 * @param c the class of objects to show instance owners for.
 */
void show_instance_owners (hashtab* objects, cls* c) {
}

/** method used to show a window with the owners to the given object. 
 *  This method does nothing.
 * @param objects the hash table of objects to look in.
 * @param obj the object to show instance owners for.
 */
void show_owner_for_object (hashtab* objects, obj* o) {
}

/** method used to show owner statistics.
 *  This method does nothing.
 * @param objects the hash table of objects to look in.
 * @param c the class of objects to show instance owners for.
 */
void show_owners_statistics (hashtab* objects, cls* c) {
}

/** Show the objects that are alloced by the given method.
 *  This method does nothing.
 */
void show_objects_alloced_by_method (hashtab* objects, method* m) {
}

/** Show the objects that are alloced by the given method. 
 *  This method does nothing.
*/
void show_objects_alloced_by_class (hashtab* objects, cls* c) {
}

/** Check if we have any ui-handling to do. */
int events_pending (void) {
    return 0;
}

/** Update the ui-toolkit (gtk_main_iteration ()) */
int ui_iteration (void) {
    return 0;
}

void show_deadlock (visited_threads* vt) {
    while (vt != NULL) {
	fprintf (stdout, "%s holding %s (%p)", 
		 jthread_get_thread_name (vt->mi->owner), 
		 vt->mi->name, vt->mi->id); 
	if (vt->next) 
	    fprintf (stdout, ", trying to enter %s (%p)\n", 
		     vt->next->mi->name, vt->next->mi->id);
	else 
	    fprintf (stdout, "\n");
	vt = vt->next;
    }
}

void notify_profile_window_begin_ui (void) { }
void notify_profile_window_end_ui (void) { }
void notify_profile_window_close_ui (void) { }
void notify_jvm_shutdown_ui (void) { }
void notify_update_full_ui (void) { }

/* Emacs Local Variables: */
/* Emacs mode:C */
/* Emacs c-indentation-style:"gnu" */
/* Emacs c-hanging-braces-alist:((brace-list-open)(brace-entry-open)(defun-open after)(substatement-open after)(block-close . c-snug-do-while)(extern-lang-open after)) */
/* Emacs c-cleanup-list:(brace-else-brace brace-elseif-brace space-before-funcall) */
/* Emacs c-basic-offset:4 */
/* Emacs End: */
